<template>
  <div class="tree-menu" :style="`max-height:${height - 200}px;`">
    <div class="devArea" v-if="showDevArea">
      <swiper :options="swiperOption">
        <swiper-slide
          v-for="item in nodeList"
          :key="item.devId"
          :class="{ active: activeTabObj.devId === item.devId }"
        >
          <div class="tab" @click="chooseTab(item)">
            {{ item.devName }}
          </div>
        </swiper-slide>
      </swiper>
      <div class="swiper-pre-border" v-if="showDevArea">
        <div class="icon-btn" slot="button-prev">
          <i class="iconfont">&#xe618;</i>
        </div>
      </div>
      <div class="swiper-next-border" v-if="showDevArea">
        <div class="icon-btn" slot="button-next">
          <i class="iconfont">&#xe623;</i>
        </div>
      </div>
    </div>
    <z-tree
      :nodes="node"
      :show-checkbox="TreeDataPool.multiple"
      :readonly="TreeDataPool.readonly"
      :gb28181="gb28181"
      :setting="setting"
      @onCreated="handleCreated"
      @onClick="itemClick"
      @onCheck="itemCheck"
      @onDblClick="onDblClick"
      @onAddNode="addNode"
      @onRemoveNode="delNode"
      @onRenameNode="editNode"
      @onAddDevice="addCamera"
      @onImport="importCameras"
      @onExpand="itemExpand"
      @onCollapse="itemCollapse"
      @onShowPic="showCameraPic"
      :search="search"
    />
    <div class="empty" v-if="!node.length">暂无摄像机数据,请添加摄像机!</div>
    <div class="dialog-box-bg" v-show="showDialog" @click="hideDialogBox"></div>
    <div class="dialog-box" v-show="showDialog">
      <el-card :body-style="{ padding: '10px' }">
        <el-form
          :model="dialogForm"
          size="mini"
          :rules="rules"
          ref="dialogForm"
          label-width="70px"
        >
          <el-form-item label="名称：" prop="name">
            <el-input
              v-model="dialogForm.text"
              oninput="if(value.length>10)value=value.slice(0,10)"
            ></el-input>
          </el-form-item>
          <div class="text-center pb-2">
            <el-button size="mini" type="primary" @click="submitForm"
              >保存</el-button
            >
            <el-button size="mini" @click="hideDialogBox">取消</el-button>
          </div>
        </el-form>
      </el-card>
    </div>
  </div>
</template>

<script>
import ZTree from "./zTree/ztree";

export default {
  name: "GiantTreeMenu",
  components: {
    ZTree,
  },
  props: {
    app: {
      type: String,
      default: "Video",
    },
    node: {
      type: Array,
    },
    treeName: {
      type: String,
      default: "",
    },
    gb28181: {
      type: Boolean,
      default: false,
    },
    height: {
      type: Number,
      default: 0,
    },
    setting: {
      type: Object,
      require: false,
      default: function () {
        return {};
      },
    },
    search: {
      type: Boolean,
      default: false,
    },
    showDevArea: {
      type: Boolean,
      default: false,
    },
    nodeList: {},
  },
  data() {
    return {
      showDialog: false,
      clientX: 0,
      clientY: 0,
      dialogForm: {
        text: "",
      },
      rules: {
        text: [
          { required: true, message: "请输入节点名称", trigger: "change" },
          { min: 2, max: 10, message: "长度在2到10个字", trigger: "change" },
        ],
      },
      curNodeTid: "",

      activeTabObj: {},
      swiperOption: {
        slidesPerView: 3,
        spaceBetween: 0,
        pagination: {
          el: ".swiper-pagination",
          clickable: true,
        },
        navigation: {
          nextEl: ".swiper-next-border",
          prevEl: ".swiper-pre-border",
        },
        observer: true, //修改swiper自己或子元素时，自动初始化swiper
        observeParents: true, //修改swiper的父元素时，自动初始化swiper
      },
    };
  },
  created() {
    this.activeTabObj =
      this.nodeList && this.nodeList.length > 0 ? this.nodeList[0] : "";
    this.TreeDataPool.zTree = true;
    this.TreeDataPool.activeVideoIndex = sessionStorage.activeIndexVideo
      ? Number(sessionStorage.activeIndexVideo)
      : this.TreeDataPool.activeVideoIndex;
  },
  watch: {
    "TreeDataPool.treeType": function (newValue) {
      if (newValue !== this.treeName) {
        this.TreeDataPool.cleanTree(this.treeName);
      }
    },
    showDevArea: function () {
      this.$forceUpdate();
    },
  },
  methods: {
    handleCreated: function (ztreeObj) {
      let _this = this;
      this.ztreeObj = ztreeObj;
      ztreeObj.expandAll(true);
      // onCreated 中操作ztreeObj对象展开第一个节点
      // ztreeObj.expandNode(ztreeObj.getNodes()[0], true);

      // 摄像机配置切换时, 设置选中状态
      if (this.TreeDataPool.selectedNode.id) {
        // 取消所有多选的节点,仅保留当前单选的节点
        this.TreeDataPool.selectedNodes = [this.TreeDataPool.selectedNode.id];
        this.curNodeTid = this.TreeDataPool.selectedNode.tId;
        //摄像机信息更新信息后，如果节点位置有变tId就不准了,this.TreeDataPool.selectedNode此时还是旧的信息
        let ztreeNodes = ztreeObj.getNodes();
        //var curNodeTid = '';
        _this.findTidByIdFromArr(ztreeNodes);
        this.TreeDataPool.selectedNode.tId = _this.curNodeTid;
        let node = this.ztreeObj.getNodeByTId(
          this.TreeDataPool.selectedNode.tId
        );

        // 多选时, 选中单选单击的节点
        if (this.TreeDataPool.multiple) {
          this.ztreeObj.checkAllNodes(false);
          this.ztreeObj.checkNode(node, true, false, false);
        }

        this.ztreeObj.selectNode(node, false, true);
      }
    },
    chooseTab(item) {
      this.$emit("saveTree", this.activeTabObj);
      setTimeout(() => {
        this.activeTabObj = item;
      }, 100);
    },

    findTidByIdFromArr(arr) {
      let len = arr.length;
      let _this = this;
      for (var i = 0; i < len; i++) {
        if (arr[i].id == this.TreeDataPool.selectedNode.id) {
          _this.curNodeTid = arr[i].tId;
          break;
        }
        if (arr[i].children) {
          this.findTidByIdFromArr(arr[i].children);
        }
      }
    },
    onDblClick(evt, treeId, item) {
      if (item.type !== "4" || this.app !== "Camera") {
        return;
      }
      this.TreeDataPool.activeVideoId = item.id;
      let videoArr = this.TreeDataPool.videoArr;
      let nullVideoIndex = "";

      if (
        this.TreeDataPool.activeForceChoose &&
        this.TreeDataPool.activeVideoIndex !== "" &&
        this.TreeDataPool.activeVideoIndex <= videoArr.length - 1
      ) {
        this.TreeDataPool.setVideoArr(
          this.TreeDataPool.activeVideoIndex,
          undefined,
          this
        );
        this.$nextTick(() => {
          this.TreeDataPool.setVideoArr(
            this.TreeDataPool.activeVideoIndex,
            item,
            this
          );
        });
        return;
      }

      for (let i = 0; i < videoArr.length; i++) {
        // eslint-disable-next-line
        if (
          videoArr[i] === "" ||
          videoArr[i] === undefined ||
          videoArr[i] === null
        ) {
          nullVideoIndex = i;
          break;
        } else {
          nullVideoIndex = "";
        }
      }
      if (nullVideoIndex === "") {
        this.TreeDataPool.setVideoArr(
          this.TreeDataPool.activeVideoIndex,
          undefined,
          this
        );
        this.$nextTick(() => {
          this.TreeDataPool.setVideoArr(
            this.TreeDataPool.activeVideoIndex,
            item,
            this
          );
        });
      } else {
        // this.TreeDataPool.setVideoArr(this.TreeDataPool.activeVideoIndex, undefined, this);
        this.$nextTick(() => {
          this.TreeDataPool.setVideoArr(
            this.TreeDataPool.activeVideoIndex,
            item,
            this
          );
        });
        this.TreeDataPool.activeVideoIndex = nullVideoIndex;
      }
    },
    addCamera(node) {
      this.$emit("addDevice", node);
    },
    importCameras(node) {
      this.$emit("import", node);
    },
    addNode(node) {
      this.dialogForm = {
        text: "",
        method: "add",
        node: node.id,
      };
      this.showDialogBox(event);
    },
    editNode(node) {
      this.dialogForm = {
        text: node.name,
        method: "edit",
        node: node.id,
        alias: node.alias,
        gb28181: this.gb28181,
      };
      this.showDialogBox(event);
    },
    delNode(node) {
      this.TreeDataPool.del(node.id);
    },
    submitForm() {
      // 提交新增或者编辑区域节点表单
      this.$refs.dialogForm.validate((valid) => {
        if (valid) {
          if (this.dialogForm.method == "add") {
            this.TreeDataPool.add(this.dialogForm.text, this.dialogForm.node);
          } else if (this.dialogForm.method == "edit") {
            this.TreeDataPool.update(
              this.dialogForm.text,
              this.dialogForm.node,
              this.gb28181 ? this.dialogForm.text : "",
              this.dialogForm.gb28181
            );
          }
        } else {
          return false;
        }
      });
      this.hideDialogBox();
    },
    hideDialogBox() {
      this.showDialog = false;
      this.dialogForm = { text: "" };
    },
    showDialogBox(event) {
      let { clientX = 0, offsetY = 0 } = event;
      // this.clientX = clientX - 120;
      this.clientX = 50;
      this.clientY = offsetY;
      this.showDialog = true;
    },
    itemClick(evt, treeId, treeNode) {
      console.log("click");
      this.TreeDataPool.selectedNode = treeNode;
      this.TreeDataPool.activeNode = treeNode;
      this.TreeDataPool.treeType = this.treeName;

      // 多选
      if (this.TreeDataPool.multiple) {
        // 单击某一个节点文字时 取消所有勾选状态 然后选中自身
        this.ztreeObj.checkAllNodes(false);
        this.ztreeObj.checkNode(treeNode, true, false, false);
      }

      this.TreeDataPool.updateZTreeCheckNodes([treeNode]);
    },
    // itemClick(treeNode) {
    //   this.TreeDataPool.selectedNode = treeNode;
    //   this.TreeDataPool.treeType = this.treeName;

    //   // 多选
    //   if (this.TreeDataPool.multiple) {
    //     // 单击某一个节点文字时 取消所有勾选状态 然后选中自身
    //     this.ztreeObj.checkAllNodes(false);
    //     this.ztreeObj.checkNode(treeNode, true, false, false);
    //   }

    //   this.TreeDataPool.updateZTreeCheckNodes([treeNode]);
    // },
    itemCheck(evt, treeId, treeNode) {
      if (treeNode) {
        this.TreeDataPool.selectedNode = treeNode;
      }
      this.TreeDataPool.treeType = this.treeName;

      // 多选
      // this.ztreeObj.checkNode(treeNode, true, false, false);
      let checkedNodes = this.ztreeObj.getCheckedNodes(true);
      this.TreeDataPool.updateZTreeCheckNodes(checkedNodes);

      // 实时统计选中个数
      this.TreeDataPool.countCheckedNodes(checkedNodes);

      // 保存一份数据
      this.TreeDataPool.activeTreeData = this.ztreeObj.getNodes();
      this.$emit("itemChecked", treeNode);
    },
    //展开
    itemExpand(e, id, node) {
      delete this.TreeDataPool.foldNodeList[node.id];
    },
    // 折叠
    itemCollapse(e, id, node) {
      this.TreeDataPool.foldNodeList[node.id] = true;
    },
    dropNode(node, item, draggedItem, e) {
      // console.log('dropNode', node, item, draggedItem);
      this.TreeDataPool.dropNode(draggedItem.id, item.id);
    },
    showCameraPic(nodeId) {
      this.TreeDataPool.showBaseImage(nodeId);
    },
  },
};
</script>
<style lang="scss" scoped>
.devArea {
}

.dialog-box {
  top: 415px;
  left: 43px;
  position: fixed;
  width: 220px;
  z-index: 3;
}
.dialog-box-bg {
  position: absolute;
  top: 0;
  left: 0;
  bottom: 0;
  right: 0;
  background: rgba(255, 255, 255, 0.6);
}
.tree-item-handle {
  display: inline;
  font-size: 16px;
  margin-left: 6px;
  button {
    position: relative;
    color: #3d68e1;
    border: 0px;
    background-color: transparent;
    cursor: pointer;
    outline: none;
  }
  button:hover {
    color: #2249b4;
  }
  // button:hover:after {
  //   position: fixed;
  //   padding: 5px 10px;
  //   background-color: #00000090;
  //   border-radius: 0px;
  //   color: #fff;
  //   content: attr(labelTooltip);
  //   font-size: 12px;
  //   width: 55px;
  //   margin-left: -50px;
  //   margin-top: 25px;
  //   vertical-align: left;
  // }
}
.tree-name {
  font-family: PingFangSC-Medium;
  font-size: 13px;
  color: #222222;
  // max-width: 100px;
  line-height: 27px;
  margin-left: 4px;
  display: inline-block;
  text-overflow: ellipsis;
  white-space: nowrap;
  overflow: hidden;
  vertical-align: top;
}
.tree-menu {
  // max-width: 350px;
  min-height: 92px;
  overflow-x: auto;
  overflow-y: hidden;
  margin-bottom: 4px;
}
.empty {
  line-height: 76px;
}
.tree-menu::-webkit-scrollbar {
  /*滚动条整体样式*/
  width: 4px; /*高宽分别对应横竖滚动条的尺寸*/
  height: 4px;
}
.tree-menu::-webkit-scrollbar-thumb {
  /*滚动条里面小方块*/
  border-radius: 5px;
  -webkit-box-shadow: inset 0 0 5px rgba(157, 165, 183, 0);
  background: rgb(202, 201, 201);
}
.tree-menu::-webkit-scrollbar-track {
  /*滚动条里面轨道*/
  -webkit-box-shadow: inset 0 0 5px rgba(157, 165, 183, 0);
  border-radius: 0;
  background: rgb(235, 234, 234);
}
.tree-menu::-webkit-scrollbar-thumb:hover {
  background: rgba(0, 0, 0, 0.4);
}
.tree-menu:hover {
  overflow-y: auto;
}

.devArea {
  position: relative;
  margin: 20px;
  background: #f0f5fa;
  padding: 0 40px;

  .swiper-slide {
    display: flex;
    align-items: center;
    height: 46px;

    &.active {
      .tab {
        color: #0064ff;
      }
      border-bottom: 2px solid #0064ff;
    }

    .tab {
      margin: 0 12px 0 24px;
      font-size: 14px;
      color: #666;
      cursor: pointer;
    }

    .iconfont {
      font-size: 12px;
      color: #dbdbdb;
    }
  }

  .swiper-pre-border {
    position: absolute;
    padding-bottom: 13px;
    padding-right: 18px;
    top: 12px;
    left: 10px;
    cursor: pointer;

    i {
      font-size: 16px;
      color: #999;
    }
  }

  .swiper-next-border {
    position: absolute;
    top: 12px;
    right: 10px;
    padding-bottom: 13px;
    padding-left: 18px;
    cursor: pointer;

    i {
      font-size: 16px;
      color: #999;
    }
  }

  .swiper-button-disabled {
    cursor: not-allowed;
    i {
      color: #dbdbdb;
    }
  }

  .del {
    cursor: pointer;
  }
}
</style>
